@@ -1,5 +1,5 @@
 # Import tableone objects
-from .tableone import TableOne, load_dataset
+from .tableone import TableOne, load_dataset, tableone
 
 __author__ = "Tom Pollard <tpollard@mit.edu>, Alistair Johnson, Jesse Raffa"
-__version__ = "0.7.6"
\ No newline at end of file
+__version__ = "0.7.10"
