@@ -2,7 +2,7 @@
 The tableone package is used for creating "Table 1" summary statistics for
 research papers.
 """
-
+from typing import Callable, Optional, Union
 import warnings
 
 import numpy as np
@@ -18,7 +18,7 @@ from tableone.modality import hartigan_diptest
 warnings.simplefilter('always', DeprecationWarning)
 
 
-def load_dataset(name):
+def load_dataset(name: str):
     """
     Load an example dataset from the online repository (requires internet).
 
@@ -43,6 +43,16 @@ def load_dataset(name):
     return df
 
 
+def docstring_copier(*sub):
+    """
+    Wrap the TableOne docstring (not ideal :/)
+    """
+    def dec(obj):
+        obj.__doc__ = obj.__doc__.format(*sub)
+        return obj
+    return dec
+
+
 class InputError(Exception):
     """
     Exception raised for errors in the input.
@@ -82,7 +92,11 @@ class TableOne(object):
         Display computed P-Values (default: False).
     pval_adjust : str, optional
         Method used to adjust P-Values for multiple testing.
-        For a complete list, see documentation for statsmodels multipletests.
+        The P-values from the unadjusted table (default when pval=True)
+        are adjusted to account for the number of total tests that were performed.
+        These adjustments would be useful when many variables are being screened
+        to assess if their distribution varies by the variable in the groupby argument.
+        For a complete list of methods, see documentation for statsmodels multipletests.
         Available methods include ::
 
         `None` : no correction applied.
@@ -118,9 +132,6 @@ class TableOne(object):
     order : dict, optional
         Specify an order for categorical variables. Key is the variable, value
         is a list of values in order.  {e.g. 'sex': ['f', 'm', 'other']}
-    remarks : bool, optional
-        Add remarks on the appropriateness of the summary measures and the
-        statistical tests (default: True).
     label_suffix : bool, optional
         Append summary type (e.g. "mean (SD); median [Q1,Q3], n (%); ") to the
         row label (default: True).
@@ -131,12 +142,28 @@ class TableOne(object):
         variables. For continuous variables, applies to all summary statistics
         (e.g. mean and standard deviation). For categorical variables, applies
         to percentage only.
-    overall : bool:
+    overall : bool, optional
         If True, add an "overall" column to the table. Smd and p-value
         calculations are performed only using stratified columns.
-    display_all : bool:
+    row_percent : bool, optional
+        If True, compute "n (%)" percentages for categorical variables across
+        "groupby" rows rather than columns.
+    display_all : bool, optional
         If True, set pd. display_options to display all columns and rows.
         (default: False)
+    dip_test : bool, optional
+        Run Hartigan's Dip Test for multimodality. If variables are found to
+        have multimodal distributions, a remark will be added below the Table 1.
+        (default: False)
+    normal_test : bool, optional
+        Test the null hypothesis that a sample come from a normal distribution.
+        Uses scipy.stats.normaltest. If variables are found to have non-normal
+        distributions, a remark will be added below the Table 1.
+        (default: False)
+    tukey_test : bool, optional
+        Run Tukey's test for far outliers. If variables are found to
+        have far outliers, a remark will be added below the Table 1.
+        (default: False)
 
     Attributes
     ----------
@@ -168,14 +195,23 @@ class TableOne(object):
 
     ...
     """
-
-    def __init__(self, data, columns=None, categorical=None, groupby=None,
-                 nonnormal=None, min_max=None, pval=False, pval_adjust=None,
-                 htest_name=False, pval_test_name=False, htest=None,
-                 isnull=None, missing=True, ddof=1, labels=None, rename=None,
-                 sort=False, limit=None, order=None, remarks=True,
-                 label_suffix=True, decimals=1, smd=False, overall=True,
-                 display_all=False):
+    def __init__(self, data: pd.DataFrame, columns: Optional[list] = None,
+                 categorical: Optional[list] = None,
+                 groupby: Optional[str] = None,
+                 nonnormal: Optional[list] = None,
+                 min_max: Optional[list] = None, pval: Optional[bool] = False,
+                 pval_adjust: Optional[str] = None, htest_name: bool = False,
+                 pval_test_name: bool = False, htest: Optional[dict] = None,
+                 isnull: Optional[bool] = None, missing: bool = True,
+                 ddof: int = 1, labels: Optional[dict] = None,
+                 rename: Optional[dict] = None, sort: Union[bool, str] = False,
+                 limit: Union[int, dict, None] = None,
+                 order: Optional[dict] = None, remarks: bool = False,
+                 label_suffix: bool = True, decimals: Union[int, dict] = 1,
+                 smd: bool = False, overall: bool = True,
+                 row_percent: bool = False, display_all: bool = False,
+                 dip_test: bool = False, normal_test: bool = False,
+                 tukey_test: bool = False) -> None:
 
         # labels is now rename
         if labels is not None and rename is not None:
@@ -203,6 +239,19 @@ class TableOne(object):
         else:
             self._pval_test_name = htest_name
 
+        # remarks are now specified by individual test names
+        if remarks:
+            warnings.warn("The remarks argument is deprecated; specify tests "
+                          "by name instead (e.g. diptest = True)",
+                          DeprecationWarning)
+            self._dip_test = remarks
+            self._normal_test = remarks
+            self._tukey_test = remarks
+        else:
+            self._dip_test = dip_test
+            self._normal_test = normal_test
+            self._tukey_test = tukey_test
+
         # groupby should be a string
         if not groupby:
             groupby = ''
@@ -222,7 +271,12 @@ class TableOne(object):
 
         # if the input dataframe is empty, raise error
         if data.empty:
-            raise InputError("The input dataframe is empty.")
+            raise InputError("Input data is empty.")
+
+        # if the input dataframe has a non-unique index, raise error
+        if not data.index.is_unique:
+            raise InputError("Input data contains duplicate values in the "
+                             "index. Reset the index and try again.")
 
         # if columns are not specified, use all columns
         if not columns:
@@ -237,7 +291,7 @@ class TableOne(object):
         # check for duplicate columns
         dups = data[columns].columns[data[columns].columns.duplicated()].unique()
         if not dups.empty:
-            raise InputError("""Input contains duplicate
+            raise InputError("""Input data contains duplicate
                                 columns: {}""".format(dups))
 
         # if categorical not specified, try to identify categorical
@@ -292,11 +346,11 @@ class TableOne(object):
         self._ddof = ddof
         self._limit = limit
         self._order = order
-        self._remarks = remarks
         self._label_suffix = label_suffix
         self._decimals = decimals
         self._smd = smd
         self._overall = overall
+        self._row_percent = row_percent
 
         # display notes and warnings below the table
         self._warnings = {}
@@ -446,7 +500,7 @@ class TableOne(object):
         when interpreting the summary statistics.
         """
         # generate warnings for continuous variables
-        if self._continuous:
+        if self._continuous and self._tukey_test:
             # highlight far outliers
             outlier_mask = self.cont_describe.far_outliers > 1
             outlier_vars = list(self.cont_describe.far_outliers[outlier_mask].
@@ -455,21 +509,23 @@ class TableOne(object):
                 self._warnings["""Tukey test indicates far outliers
                                   in"""] = outlier_vars
 
+        if self._continuous and self._dip_test:
             # highlight possible multimodal distributions using hartigan's dip
             # test -1 values indicate NaN
-            modal_mask = ((self.cont_describe.diptest >= 0) &
-                          (self.cont_describe.diptest <= 0.05))
-            modal_vars = list(self.cont_describe.diptest[modal_mask].
+            modal_mask = ((self.cont_describe.hartigan_dip >= 0) &
+                          (self.cont_describe.hartigan_dip <= 0.05))
+            modal_vars = list(self.cont_describe.hartigan_dip[modal_mask].
                               dropna(how='all').index)
             if modal_vars:
                 self._warnings["""Hartigan's Dip Test reports possible
                                   multimodal distributions for"""] = modal_vars
 
+        if self._continuous and self._normal_test:
             # highlight non normal distributions
             # -1 values indicate NaN
-            modal_mask = ((self.cont_describe.normaltest >= 0) &
-                          (self.cont_describe.normaltest <= 0.001))
-            modal_vars = list(self.cont_describe.normaltest[modal_mask].
+            modal_mask = ((self.cont_describe.normality >= 0) &
+                          (self.cont_describe.normality <= 0.001))
+            modal_vars = list(self.cont_describe.normality[modal_mask].
                               dropna(how='all').index)
             if modal_vars:
                 self._warnings["""Normality test reports non-normal
@@ -688,7 +744,7 @@ class TableOne(object):
         else:
             return np.nanstd(x.values, ddof=self._ddof)
 
-    def _diptest(self, x):
+    def _hartigan_dip(self, x):
         """
         Compute Hartigan Dip Test for modality.
 
@@ -702,7 +758,7 @@ class TableOne(object):
             return -1
         return p
 
-    def _normaltest(self, x):
+    def _normality(self, x):
         """
         Compute test for normal distribution.
 
@@ -820,9 +876,17 @@ class TableOne(object):
                 Summarise the continuous variables.
         """
         aggfuncs = [pd.Series.count, np.mean, np.median, self._std,
-                    self._q25, self._q75, min, max, self._t1_summary,
-                    self._diptest, self._outliers, self._far_outliers,
-                    self._normaltest]
+                    self._q25, self._q75, min, max, self._t1_summary]
+
+        if self._dip_test:
+            aggfuncs.append(self._hartigan_dip)
+
+        if self._tukey_test:
+            aggfuncs.append(self._outliers)
+            aggfuncs.append(self._far_outliers)
+
+        if self._normal_test:
+            aggfuncs.append(self._normality)
 
         # coerce continuous data to numeric
         cont_data = data[self._continuous].apply(pd.to_numeric,
@@ -867,14 +931,14 @@ class TableOne(object):
 
         return df_cont
 
-    def _format_cat(self, row):
+    def _format_cat(self, row, col):
         var = row.name[0]
         if var in self._decimals:
             n = self._decimals[var]
         else:
             n = 1
         f = '{{:.{}f}}'.format(n)
-        return f.format(row.percent)
+        return f.format(row[col])
 
     def _create_cat_describe(self, data, groupby, groupbylvls):
         """
@@ -884,6 +948,10 @@ class TableOne(object):
         ----------
             data : pandas DataFrame
                 The input dataset.
+            groupby : Str
+                Variable to group by.
+            groupbylvls : List
+                List of levels in the groupby variable.
 
         Returns
         ----------
@@ -892,46 +960,64 @@ class TableOne(object):
         """
         group_dict = {}
 
+        cat_slice = data[self._categorical].copy()
+
         for g in groupbylvls:
             if groupby:
-                d_slice = data.loc[data[groupby] == g, self._categorical]
+                df = cat_slice.loc[data[groupby] == g, self._categorical]
             else:
-                d_slice = data[self._categorical].copy()
+                df = cat_slice.copy()
 
-            # create a dataframe with freq, proportion
-            df = d_slice.copy()
+            # create n column and null count column
+            # must be done before converting values to strings
+            ct = df.count().to_frame(name='n')
+            ct.index.name = 'variable'
+            nulls = df.isnull().sum().to_frame(name='Missing')
+            nulls.index.name = 'variable'
 
-            # convert to str to handle int converted to boolean. Avoid nans.
+            # Convert to str to handle int converted to boolean in the index.
+            # Also avoid nans.
             for column in df.columns:
                 df[column] = [str(row) if not pd.isnull(row)
                               else None for row in df[column].values]
+                cat_slice[column] = [str(row) if not pd.isnull(row)
+                                     else None for row
+                                     in cat_slice[column].values]
 
+            # create a dataframe with freq, proportion
             df = df.melt().groupby(['variable',
                                     'value']).size().to_frame(name='freq')
 
             df['percent'] = df['freq'].div(df.freq.sum(level=0),
                                            level=0).astype(float) * 100
 
+            # add row percent
+            df['percent_row'] = df['freq'].div(cat_slice[self._categorical]
+                                               .melt()
+                                               .groupby(['variable', 'value'])
+                                               .size()) * 100
+
             # set number of decimal places for percent
             if isinstance(self._decimals, int):
                 n = self._decimals
                 f = '{{:.{}f}}'.format(n)
                 df['percent_str'] = df['percent'].astype(float).map(f.format)
+                df['percent_row_str'] = df['percent_row'].astype(float).map(f.format)
             elif isinstance(self._decimals, dict):
-                df.loc[:, 'percent_str'] = df.apply(self._format_cat, axis=1)
+                df.loc[:, 'percent_str'] = df.apply(self._format_cat, axis=1,
+                                                    args=['percent'])
+                df.loc[:, 'percent_row_str'] = df.apply(self._format_cat,
+                                                        axis=1,
+                                                        args=['percent_row'])
             else:
                 n = 1
                 f = '{{:.{}f}}'.format(n)
                 df['percent_str'] = df['percent'].astype(float).map(f.format)
+                df['percent_row_str'] = df['percent_row'].astype(float).map(f.format)
 
-            # add n column, listing total non-null values for each variable
-            ct = d_slice.count().to_frame(name='n')
-            ct.index.name = 'variable'
+            # join count column
             df = df.join(ct)
 
-            # add null count
-            nulls = d_slice.isnull().sum().to_frame(name='Missing')
-            nulls.index.name = 'variable'
             # only save null count to the first category for each variable
             # do this by extracting the first category from the df row index
             levels = df.reset_index()[['variable',
@@ -943,8 +1029,12 @@ class TableOne(object):
             df = df.join(nulls)
 
             # add summary column
-            df['t1_summary'] = (df.freq.map(str) + ' ('
-                                + df.percent_str.map(str)+')')
+            if self._row_percent:
+                df['t1_summary'] = (df.freq.map(str) + ' ('
+                                    + df.percent_row_str.map(str)+')')
+            else:
+                df['t1_summary'] = (df.freq.map(str) + ' ('
+                                    + df.percent_str.map(str)+')')
 
             # add to dictionary
             group_dict[g] = df
@@ -1523,3 +1613,11 @@ class TableOne(object):
         msg = ("'{}' has all non-numeric values. Consider including "
                "it in the list of categorical variables.").format(c)
         warnings.warn(msg, RuntimeWarning, stacklevel=2)
+
+
+# Allow TableOne to be called as a function.
+# Refactor this out at some point!
+@docstring_copier(TableOne.__doc__)
+def tableone(*args, **kwargs):
+    """{0}"""
+    return TableOne(*args, **kwargs)
